@@ -94,7 +94,7 @@ my $ModName;        # name of the module
 my @ModFiles;       # the .PMs in this package
 {   print "Creating top level dir..." if $Verbose;
 
-    ### make sure we get the shortest file, so we dont accidentally get
+    ### make sure we get the shortest file, so we don't accidentally get
     ### a subdir
     @ModFiles   =  sort { length($a) <=> length($b) }
                    map  { chomp; $_ }
@@ -204,7 +204,7 @@ my @ChangedFiles;
 
         ### fix installperl, so these files get installed by other utils
         ### ./installperl:    return if $name =~
-        ### /^(?:cpan|instmodsh|prove|corelist|ptar|ptardiff|config_data)\z/;
+        ### /^(?:cpan|instmodsh|prove|corelist|ptar|ptardiff)\z/;
         {   my $file = 'installperl';
 
             ### not there already?
@@ -283,8 +283,8 @@ my @ChangedFiles;
             push @NewFiles, $file;
         }
 
-        ### add an entry to utils/Makefile.SH for $bin
-        {   my $file = "utils/Makefile.SH";
+        ### add an entry to utils/Makefile.PL for $bin
+        {   my $file = "utils/Makefile.PL";
 
             ### not there already?
             unless( `grep $bin $Repo/$file` ) {
@@ -344,7 +344,7 @@ my @ChangedFiles;
         }
 
         ### we need some entries in a vms specific file as well..
-        ### except, i dont understand how it works or what it does, and it
+        ### except, I don't understand how it works or what it does, and it
         ### looks all a bit odd... so lets just print a warning...
         ### the entries look something like this:
         # ./vms/descrip_mms.template:utils4 = [.utils]enc2xs.com
@@ -358,8 +358,8 @@ my @ChangedFiles;
                 print $/.$/;
                 print "    WARNING! You should add entries like the following\n"
                     . "    to $file (Using $TestBin as an example)\n"
-                    . "    Unfortunately I dont understand what these entries\n"
-                    . "    do, so I wont change them automatically:\n\n";
+                    . "    Unfortunately I don't understand what these entries\n"
+                    . "    do, so I won't change them automatically:\n\n";
 
                 print `grep -nC1 $TestBin $Repo/$file`;
                 print $/.$/;
@@ -371,49 +371,6 @@ my @ChangedFiles;
     }
 }
 
-### binary files must be encoded!
-### XXX use the new 'uupacktool.pl'
-{   my $pack = "$Repo/uupacktool.pl";
-
-    ### pack.pl encodes binary files for us
-    -e $pack or die "Need $pack to encode binary files!";
-
-    ### chdir, so uupacktool writes relative files properly
-    ### into it's header...
-    my $curdir = cwd();
-    chdir($Repo) or die "Could not chdir to '$Repo': $!";
-
-    for my $aref ( \@ModFiles, \@TestFiles, \@BinFiles ) {
-        for my $file ( @$aref ) {
-            my $full = -e $file                 ? $file              :
-                       -e "$RelTopDir/$file"    ? "$RelTopDir/$file" :
-                       die "Can not find $file in $Repo or $TopDir\n";
-
-            if( -f $full && -s _ && -B _ ) {
-                print "Binary file $file needs encoding\n" if $Verbose;
-
-                my $out = $full . '.packed';
-
-                ### does the file exist already?
-                ### and doesn't have +w
-                if( -e $out && not -w _ ) {
-                    system("chmod +w $out")
-                        and die "Could not set chmod +w to '$out': $!";
-                }
-
-                ### -D to remove the original
-                system("$^X $pack -D -p $full $out")
-                    and die "Could not encode $full to $out";
-
-
-                $file .= '.packed';
-            }
-        }
-    }
-
-    chdir($curdir) or die "Could not chdir back to '$curdir': $!";
-}
-
 ### update the manifest
 {   my $file        = $Repo . '/MANIFEST';
     my @manifest;
